/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands {
import com.adobe.cairngorm.business.ServiceLocator;
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;

import mx.collections.ArrayCollection;
import mx.rpc.AsyncToken;
import mx.rpc.Responder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
import mx.rpc.remoting.RemoteObject;

import org.ow2.orchestra.designer.events.ProcessEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.utils.ErrorManager;
import org.ow2.orchestra.designer.vo.FileNavigatorProcess;

public class CloseProcessCMD implements ICommand {

  public function CloseProcessCMD() {
  }

  public function execute(event:CairngormEvent):void {
    var processEvent:ProcessEvent = event as ProcessEvent;
    var processEditorModel:ProcessEditorModel = processEvent.processEditorModel;
    var process:FileNavigatorProcess = processEditorModel.fileNavigatorProcess;
    //Remove the process from the opened processes
    var openedProcesses:ArrayCollection = DesignerModelLocator.getInstance().openedProcesses;
    var ind:int = openedProcesses.getItemIndex(processEditorModel);
    if (ind >= 0) {
      openedProcesses.removeItemAt(ind);
      if (processEditorModel == DesignerModelLocator.getInstance().selectedProcessEditor) {
        //Update the selected process
        if (openedProcesses.length > 0) {
          var newSelectedProcessEditorIndex:int;
          if (ind < openedProcesses.length) {
            newSelectedProcessEditorIndex = ind;
          }
          else {
            newSelectedProcessEditorIndex = ind - 1;
          }
          DesignerModelLocator.getInstance().selectedProcessEditor =
              openedProcesses.getItemAt(newSelectedProcessEditorIndex) as ProcessEditorModel;
        } else {
          DesignerModelLocator.getInstance().selectedProcessEditor = null;
        }
      }
    }

    if (processEditorModel.readWrite) {
      var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");
      var token:AsyncToken = remote.freeTokenForProcess(process);
      // add fonctions to call on result
      token.addResponder(new Responder(freeTokenResult, fault));
    }

  }

  private function freeTokenResult(event:ResultEvent):void {
    //Nothing to do
  }

  private function fault(event:FaultEvent):void {
    ErrorManager.getInstance().updateTitle("Error when freeing access to your process.");
    ErrorManager.getInstance().showError("Remote invoke error: " + event.message + ".\n" +
        "To free access to this process please log out.")
  }

}
}
