/**
 * @author Loic Albertin
 */
package org.ow2.orchestra.designer.commands.dataTypes {
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.data.type.SequenceType;
import org.ow2.orchestra.designer.bpmn.model.data.type.WSDLMessageModel;
import org.ow2.orchestra.designer.bpmn.model.data.type.WSDLMessagePartModel;
import org.ow2.orchestra.designer.bpmn.model.data.type.XSDElementModel;
import org.ow2.orchestra.designer.bpmn.model.process.EventModel;
import org.ow2.orchestra.designer.bpmn.model.process.EventType;
import org.ow2.orchestra.designer.events.DataTypesChangeEvent;
import org.ow2.orchestra.designer.model.ProcessEditorModel;

public class DeleteDataTypesCMD implements ICommand {
  public function DeleteDataTypesCMD() {
  }

  public function execute(event:CairngormEvent):void {
    var dataTypeEvent:DataTypesChangeEvent = event as DataTypesChangeEvent;
    if (event.type == DataTypesChangeEvent.DATA_TYPE_DELETE) {
      var processEditorModel:ProcessEditorModel = dataTypeEvent.processEditorModel;
      if (processEditorModel.selectedElements.length == 1) {
        var selectedElementModel:AbstractElement = processEditorModel.selectedElements.getItemAt(0) as AbstractElement;

        if (selectedElementModel is EventModel) {
          var selectedEvent:EventModel = selectedElementModel as EventModel;
          var selectedDataType:WSDLMessageModel;
          if (selectedEvent.eventType == EventType.RECEIVE_MESSAGE
              && (selectedEvent.operation != null && selectedEvent.operation.inMessageType != null)) {
            selectedDataType = selectedEvent.operation.inMessageType as WSDLMessageModel;
          }
          else if (selectedEvent.eventType == EventType.SEND_MESSAGE
              && (selectedEvent.operation != null && selectedEvent.operation.outMessageType != null)) {
            selectedDataType = selectedEvent.operation.outMessageType as WSDLMessageModel;
          }

          if (selectedDataType != null && selectedDataType.parts != null) {
            var partModel:WSDLMessagePartModel = selectedDataType.parts.getItemAt(0) as WSDLMessagePartModel;

            if (partModel != null) {
              var sequence:SequenceType = null;
              if (partModel.xsdType != null && partModel.xsdType is SequenceType) {
                sequence = SequenceType(partModel.xsdType);
              } else if (partModel.xsdElement != null && partModel.xsdElement.type is SequenceType) {
                sequence = SequenceType(partModel.xsdElement.type);
              }
              if (sequence != null) {
                sequence.elements.removeItemAt(sequence.elements.getItemIndex(dataTypeEvent.elementModel));
                //If there is now only one element in sequence then create a type for it and remove the sequence
                if (sequence.elements.length == 1) {
                  partModel.xsdElement = sequence.elements.getItemAt(0) as XSDElementModel;
                  partModel.xsdType = null;
                  sequence.elements.removeAll();
                }
              }
            }
          }
        }
      }
    }
  }
}
}
