/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands.editor {
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;

import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.IDiagramModel;
import org.ow2.orchestra.designer.bpmn.model.IIdentifiableElement;
import org.ow2.orchestra.designer.bpmn.model.process.AbstractBPMNElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.process.PoolModel;
import org.ow2.orchestra.designer.bpmn.model.process.ProcessModel;
import org.ow2.orchestra.designer.bpmn.model.utils.IDUtil;
import org.ow2.orchestra.designer.events.DiagramModificationEvent;
import org.ow2.orchestra.designer.utils.DesignerUtil;

public class AddDiagramElementCMD implements ICommand {
  public function AddDiagramElementCMD() {
  }

  /**
   * Executes the command.
   * The given event should be a @see org.ow2.orchestra.designer.events.DiagramModificationEvent
   * of type <code>ProcessModificationEvent.PROCESS_ELEMENT_ADD</code>containing an
   * @see org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition as model element.
   *
   * @param event a ProcessModificationEvent
   *
   */
  public function execute(event:CairngormEvent):void {
    if (event != null && event is DiagramModificationEvent) {
      var processEvt:DiagramModificationEvent = event as DiagramModificationEvent;
      if (processEvt.type == DiagramModificationEvent.DIAGRAM_ELEMENT_ADD && processEvt.diagramEditorModel != null) {
        // save process model
        processEvt.diagramEditorModel.saveModelForUndo();
        var newModel:AbstractElementWithPosition = addNewElementToModel(processEvt.element_pos_x, processEvt.element_pos_y, processEvt.modelElement, processEvt.diagramEditorModel.diagramModel);
        // selected newly added element
        processEvt.diagramEditorModel.selectedElements.removeAll();
        processEvt.diagramEditorModel.selectedElements.addItem(newModel);
      }
    }
  }

  public static function addNewElementToModel(x:int, y:int, type:AbstractElementWithPosition, diagramModel:IDiagramModel):AbstractElementWithPosition {
    //Warning: DesignerUtil.copy performs a deep copy of an object
    var newModel:AbstractElementWithPosition = (DesignerUtil.copy(type) as AbstractElementWithPosition);
    if (newModel != null) {
      if (newModel is IIdentifiableElement) {
        IIdentifiableElement(newModel).id = IDUtil.generateId();
      }
      newModel.x = x;
      newModel.y = y;
      if (diagramModel is ProcessModel && newModel is AbstractBPMNElementWithPosition) {
        var processModel:ProcessModel = diagramModel as ProcessModel;
        AbstractBPMNElementWithPosition(newModel).processModel = processModel;
        if (newModel is PoolModel) {
          newModel.x = 0;
          newModel.y = 0;
          for each (var o:Object in diagramModel.elements) {
            if (o is AbstractElementWithPosition) {
              (o as AbstractElementWithPosition).y += PoolModel.POOL_HEIGHT;
            }
          }
          processModel.y += PoolModel.POOL_HEIGHT;
        }
      }
      diagramModel.elements.addItem(newModel);
    }

    return newModel;
  }
}
}
