/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands.editor {
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;

import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.FlowModel;
import org.ow2.orchestra.designer.bpmn.model.IDiagramModel;
import org.ow2.orchestra.designer.bpmn.model.process.ProcessModel;
import org.ow2.orchestra.designer.bpmn.model.process.SequenceFlowModel;
import org.ow2.orchestra.designer.bpmn.model.utils.IDUtil;
import org.ow2.orchestra.designer.events.FlowEvent;

public class CreateFlowCMD implements ICommand {
  public function CreateFlowCMD() {
  }

  /**
   * Executes the command.
   *  @param event a FlowEvent
   */
  public function execute(event:CairngormEvent):void {
    var flowEvt:FlowEvent = event as FlowEvent;
    trace("CreateFlowCMD");

    // save process model
    flowEvt.diagramEditorModel.saveModelForUndo();
    // Create flow
    createFlow(flowEvt.sourceElement, flowEvt.targetElement, flowEvt.diagramEditorModel.diagramModel);
  }

  public static function createFlow(sourceElement:AbstractElementWithPosition, targetElement:AbstractElementWithPosition, flowModel:IDiagramModel):FlowModel {
    // Create flow
    var newFlowModel:FlowModel;
    if (flowModel is ProcessModel) {
      newFlowModel = new SequenceFlowModel();
      SequenceFlowModel(newFlowModel).id = IDUtil.generateId();
    } else {
      newFlowModel = new FlowModel();
    }
    newFlowModel.sourceElement = sourceElement;
    newFlowModel.targetElement = targetElement;

    flowModel.elements.addItem(newFlowModel);
    return newFlowModel;
  }
}
}
