/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands.editor {
  import com.adobe.cairngorm.commands.ICommand;
  import com.adobe.cairngorm.control.CairngormEvent;

  import flash.utils.describeType;

  import org.ow2.orchestra.designer.DesignerError;
  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.control.DesignerMessageCodes;
  import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.DiagramEditorModel;
import org.ow2.orchestra.designer.model.ProcessEditorModel;

  public class UpdateElementPropertyCMD implements ICommand {


    public function execute(event:CairngormEvent):void {
      var eltEvt:ElementPropertyChangeEvent = event as ElementPropertyChangeEvent;

      var selectedDiagramEditor:DiagramEditorModel = eltEvt.diagramEditorModel;
      if (selectedDiagramEditor != null) {
        // save process model for undo
        selectedDiagramEditor.saveModelForUndo();
      }

      if (hasSetter(eltEvt.modelElement, eltEvt.propertyName)) {
        eltEvt.modelElement[eltEvt.propertyName] = eltEvt.propertyValue;
      }
      else {
        throw new DesignerError(DesignerMessageCodes.UNKNOWN_PROPERTY, eltEvt.modelElement);
      }
    }

    private function hasSetter(model:AbstractElement, propertyName:String):Boolean {
      // Get the Button control's E4X XML object description.
      var classInfo:XML = describeType(model);

      // Find setter for the property.
      for each (var accessor:XML in classInfo..accessor) {
        // Do not write the property value if it is read only.
        if (accessor.@access != 'readonly' && accessor.@name == propertyName) {
          // setter for the value
          return true;
        }
      }
      return false;
    }
  }
}
