/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands.fileNavigator {
import com.adobe.cairngorm.business.ServiceLocator;
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import mx.collections.ArrayCollection;
import mx.collections.ListCollectionView;
import mx.managers.CursorManager;
import mx.rpc.AsyncToken;
import mx.rpc.Responder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
import mx.rpc.remoting.RemoteObject;

import org.ow2.orchestra.designer.control.DesignerMessageCodes;
import org.ow2.orchestra.designer.events.FileNavigatorEvent;
import org.ow2.orchestra.designer.events.ProcessEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.utils.ErrorManager;
import org.ow2.orchestra.designer.vo.FileNavigatorItem;
import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
import org.ow2.orchestra.designer.vo.FileNavigatorProject;
import org.ow2.orchestra.designer.vo.FileNavigatorUserAccessPermissions;

/**
 * Delete the selected process
 */
public class DeleteFileNavigatorItemCMD implements ICommand {
  private var item:FileNavigatorItem;
  private var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");

  public function DeleteFileNavigatorItemCMD() {
  }

  /**
   *
   * @param event FileNavigatorEvent
   *
   */
  public function execute(event:CairngormEvent):void {
    item = (event as FileNavigatorEvent).item;
    if (item is FileNavigatorProcess) {
      if ((item as FileNavigatorProcess).project.owner.name == DesignerModelLocator.getInstance().userName) {
        deleteItem();
      } else {
        var process:FileNavigatorProcess = item as FileNavigatorProcess;
        var token1:AsyncToken = remote.getAccessProperties(process);
        token1.addResponder(new Responder(getAccessPropertiesResult, fault));
      }
    } else if (item is FileNavigatorProject) {
      //Dispatch an event to delete all included processes.
      var project:FileNavigatorProject = item as FileNavigatorProject;
      var projectProcessesList:ListCollectionView = new ArrayCollection();
      projectProcessesList.addAll(project.processes);
      for each (var projectProcess:FileNavigatorProcess in projectProcessesList) {
        var closeProcessEvent:FileNavigatorEvent = new FileNavigatorEvent(FileNavigatorEvent.ITEM_DELETE, projectProcess);
        CairngormEventDispatcher.getInstance().dispatchEvent(closeProcessEvent);
      }
      // delete process
      var token2:AsyncToken = remote.deleteItem(item);
      token2.addResponder(new Responder(deleteItemResult, fault));
      CursorManager.setBusyCursor();
    }
  }

  private function getAccessPropertiesResult(event:ResultEvent):void {

    var lAccess:ArrayCollection = event.result as ArrayCollection;
    var isDeletionAllowed:Boolean = false;
    for each(var user:FileNavigatorUserAccessPermissions in lAccess) {
      if (user.user == DesignerModelLocator.getInstance().userName) {
        if (user.deleting) {
          isDeletionAllowed = true;
          break;
        }
      }
    }
    if (isDeletionAllowed) {
      deleteItem();
    } else {
      ErrorManager.getInstance().updateTitle("Warning : permission denied");
      ErrorManager.getInstance().showError("no access permission for delete item!")
    }
  }

  private function deleteItem():void {
    var processEditorModel:ProcessEditorModel = DesignerModelLocator.getInstance().getOpenProcessByFilePath(item as FileNavigatorProcess);
    if (processEditorModel != null) {
      var evt:ProcessEvent = new ProcessEvent(ProcessEvent.PROCESS_CLOSE, processEditorModel);
      CairngormEventDispatcher.getInstance().dispatchEvent(evt);
    }
    // delete process
    var token:AsyncToken = remote.deleteItem(item);
    token.addResponder(new Responder(deleteItemResult, fault));
    CursorManager.setBusyCursor();
  }

  private function deleteItemResult(event:ResultEvent):void {
    if (!event.result) {
      ErrorManager.getInstance().showError("An error occurs during process deletion.", DesignerMessageCodes.PROCESS_DELETION_ERROR);
    }
    // reload tree model
    if (item is FileNavigatorProcess) {
      new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL, (item as FileNavigatorProcess).project).dispatch();
    }
    else {
      new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL, item).dispatch();
    }
  }

  private function fault(event:FaultEvent):void {
    CursorManager.removeBusyCursor();
    ErrorManager.getInstance().showError("Remote invoke error during process deletion: " + event.message);
  }
}
}
