package org.ow2.orchestra.designer.commands.fileNavigator {
  import com.adobe.cairngorm.business.ServiceLocator;
  import com.adobe.cairngorm.commands.ICommand;
  import com.adobe.cairngorm.control.CairngormEvent;

  import mx.collections.ListCollectionView;
  import mx.managers.CursorManager;
  import mx.rpc.AsyncToken;
  import mx.rpc.Responder;
  import mx.rpc.events.FaultEvent;
  import mx.rpc.events.ResultEvent;
  import mx.rpc.remoting.RemoteObject;

  import org.ow2.orchestra.designer.events.FileNavigatorEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.model.FileNavigatorModel;
  import org.ow2.orchestra.designer.utils.ErrorManager;
  import org.ow2.orchestra.designer.vo.FileNavigatorItem;
  import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
  import org.ow2.orchestra.designer.vo.FileNavigatorProject;

  /**
   * Initialize the files list into the FileNavigatorView
   */
  public class FetchFileNavigatorTreeModelCMD implements ICommand {

    private var itemToSelect:FileNavigatorItem;

    public function FetchFileNavigatorTreeModelCMD() {
    }

    /**
     *
     * @param event
     *
     */
    public function execute(event:CairngormEvent):void {

      itemToSelect = (event as FileNavigatorEvent).item;

      var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");
      var token:AsyncToken = remote.getProjectsAndProcess();


      // add listener for the asyn response of remote object and change the cursor to busy
      token.addResponder(new Responder(getResult, fault));
    }

    private function fault(event:FaultEvent):void {
       if (event.fault.faultString != null && event.fault.faultString != "") {
        ErrorManager.getInstance().showError("Error during initialization of OrchestraDesigner: " + event.fault.faultString);
      } else {
        ErrorManager.getInstance().showError("Error during initialization of OrchestraDesigner. Please look at the server logs for more information.");
      }
    }

    private function getResult(event:ResultEvent):void {
      var fileNavigatorViewModel:FileNavigatorModel = DesignerModelLocator.getInstance().fileNavigatorModel;

      var projects:ListCollectionView = event.result as ListCollectionView;

      fileNavigatorViewModel.projects.removeAll();
      fileNavigatorViewModel.projects.addAll(projects);

      if (itemToSelect != null) {
        if (itemToSelect is FileNavigatorProject) {
          fileNavigatorViewModel.selectedItem = fileNavigatorViewModel.getProject(itemToSelect as FileNavigatorProject);
        }
        else if (itemToSelect is FileNavigatorProcess) {
          fileNavigatorViewModel.selectedItem = fileNavigatorViewModel.getProcess(itemToSelect as FileNavigatorProcess);
        }
      }

      // remove the cursor busy because the connexion with server is finished
      CursorManager.removeBusyCursor();
    }

  }
}
