package org.ow2.orchestra.designer.commands.fileNavigator {
  import com.adobe.cairngorm.business.ServiceLocator;
  import com.adobe.cairngorm.commands.ICommand;
  import com.adobe.cairngorm.control.CairngormEvent;

  import mx.managers.CursorManager;
  import mx.rpc.AsyncToken;
  import mx.rpc.Responder;
  import mx.rpc.events.FaultEvent;
  import mx.rpc.events.ResultEvent;
  import mx.rpc.remoting.RemoteObject;

  import org.ow2.orchestra.designer.bpmn.model.process.ProcessModel;
  import org.ow2.orchestra.designer.events.FileNavigatorEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.model.FileNavigatorModel;
  import org.ow2.orchestra.designer.utils.ErrorManager;
  import org.ow2.orchestra.designer.bpmn.model.utils.IDUtil;
  import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
  import org.ow2.orchestra.designer.vo.FileNavigatorProject;

  /**
   * Create a new entry in fileNavigator for the process
   */
  public class NewProcessCMD implements ICommand {

    private var process:FileNavigatorProcess;

    public function NewProcessCMD() {
    }

    private static const DEFAULT_NAMESPACE:String = "http://orchestra.ow2.org/designer/default_ns";

    /**
     * @param event FileNavigatorEvent
     */
    public function execute(event:CairngormEvent):void {
      var fileNavigatorEvent:FileNavigatorEvent = event as FileNavigatorEvent;
      process = fileNavigatorEvent.item as FileNavigatorProcess;
      var project:FileNavigatorProject = process.project;
      var processName:String = process.name;
      var fileNavigatorViewModel:FileNavigatorModel = DesignerModelLocator.getInstance().fileNavigatorModel;

      if (project != null) {
        if (fileNavigatorViewModel.processExist(project, processName)) {
          ErrorManager.getInstance().showError("This process '" + processName + "' already exists.");
        }
        else {
          var processModel:ProcessModel = new ProcessModel();
          processModel.name = process.name;
          processModel.id = IDUtil.generateId();
          processModel.wsdlNamespace = DEFAULT_NAMESPACE;
          var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");

          CursorManager.setBusyCursor();

          var token:AsyncToken = remote.saveProcess(process, processModel);
          token.addResponder(new Responder(newFileResult, fault));
        }
      } else {
        ErrorManager.getInstance().showError("Please select a project first.");
      }
    }

    private function newFileResult(event:ResultEvent):void {
      new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL, process).dispatch();
      new FileNavigatorEvent(FileNavigatorEvent.PROCESS_OPEN, process).dispatch();
    }

    private function fault(event:FaultEvent):void {
      CursorManager.removeBusyCursor();
      ErrorManager.getInstance().showError("Remote invoke error: " + event.message);
    }

  }
}

