/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.events {
  import com.adobe.cairngorm.control.CairngormEvent;

  import mx.collections.ArrayCollection;

  /**
   * @author Loic Albertin
   */
  public class ProcessEditorEvent extends CairngormEvent {
    /**
     * Save a given list of process editor models.
     */
    public static const SAVE_MODELS:String = "ProcessEditorEvent_SaveModels";

    /**
     * Export a given list of process editor models.
     */
    public static const EXPORT_MODELS:String = "ProcessEditorEvent_ExportModels";

    /**
     * Deploy a given list of process editor models to Orchestra engine.
     */
    public static const DEPLOY_MODELS:String = "ProcessEditorEvent_DeployModels";


    /**
     * The corresponding Process editor model list
     */
    private var _processEditorModelList:ArrayCollection = new ArrayCollection();

    /**
     * default constructor.
     *
     * @param type Event type
     * @param pem Process editor model list (null by default)
     * @param bubbles (false by default)
     * @param cancelable (false by default)
     */
    public function ProcessEditorEvent(type:String, pemList:ArrayCollection = null, bubbles:Boolean = false, cancelable:Boolean = false) {
      super(type, bubbles, cancelable);
      if (pemList != null) {
        _processEditorModelList = pemList;
      }
    }

    public function get processEditorModelList():ArrayCollection {
      return _processEditorModelList;
    }

  }
}
