/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.model {
import com.adobe.cairngorm.CairngormError;
import com.adobe.cairngorm.CairngormMessageCodes;
import com.adobe.cairngorm.model.IModelLocator;

import flash.net.SharedObject;

import mx.collections.ArrayCollection;

import org.ow2.orchestra.designer.vo.FileNavigatorProcess;

[Bindable]
public class DesignerModelLocator implements IModelLocator {
  /**
   * Shared Object used to persist configuration on client side
   */
  private var _persistentData:SharedObject = SharedObject.getLocal("Properties");

  private var _isPropertiesViewVisible:Boolean;

  private var _isMicroImageViewVisible:Boolean;

  private var _userName:String = "Anonymous";

  /**
   * Collection of processes currently opened.
   * Contains ProcessEditorModel elements.
   */
  private var _openedProcesses:ArrayCollection = new ArrayCollection();

  /**
   * Collection of wsdl services available.
   */
  private var _availableServices:ArrayCollection = new ArrayCollection();

  /**
   * Currently selected process ediotor.
   */
  private var _selectedProcessEditor:ProcessEditorModel = null;

  private var _fileNavigatorModel:FileNavigatorModel = new FileNavigatorModel();

  private var _orchestraConnectionProperties:OrchestraConnectionPropertiesModel;

  private var _accessProcessSettings:AccessProcessSettingsModel = new AccessProcessSettingsModel();

  private var _usersOfDesigner:ArrayCollection;

  private var _unreadNotifications:ArrayCollection;

  /**
   * Singleton Pattern.
   */
  private static var instance:DesignerModelLocator;

  public function DesignerModelLocator() {
    if (instance != null) {
      throw new CairngormError(
          CairngormMessageCodes.SINGLETON_EXCEPTION, "DesignerModelLocator");
    }

    instance = this;

    if (_persistentData.data.selected_propView == null) {
      this.isPropertiesViewVisible = false;
    } else {
      this.isPropertiesViewVisible = _persistentData.data.selected_propView;
    }

    if (_persistentData.data.selected_micro == null) {
      this.isMicroImageViewVisible = false;
    } else {
      this.isMicroImageViewVisible = _persistentData.data.selected_micro;
    }

    // Initialize Orchestra Engine connection properties
    var jmxServiceURL:String;
    var jmxObjectName:String;
    if (_persistentData.data.jmxServiceURL == null) {
      jmxServiceURL = "service:jmx:rmi:///jndi/rmi://localhost:9999/orchestraServer";
    } else {
      jmxServiceURL = _persistentData.data.jmxServiceURL;
    }
    if (_persistentData.data.jmxObjectName == null) {
      jmxObjectName = "Orchestra:type=RemoteAPI";
    } else {
      jmxObjectName = _persistentData.data.jmxObjectName;
    }
    _orchestraConnectionProperties = new OrchestraConnectionPropertiesModel(jmxServiceURL, jmxObjectName);

    _accessProcessSettings = new AccessProcessSettingsModel();

     _usersOfDesigner = new ArrayCollection();
    _unreadNotifications = new ArrayCollection();
  }

  public static function getInstance():DesignerModelLocator {
    if (instance == null) {
      instance = new DesignerModelLocator();
    }

    return instance;
  }

  public function get openedProcesses():ArrayCollection {
    return _openedProcesses;
  }

  public function get availableServices():ArrayCollection {
    return _availableServices;
  }

  public function get selectedProcessEditor():ProcessEditorModel {
    return _selectedProcessEditor;
  }

  public function set selectedProcessEditor(value:ProcessEditorModel):void {
    _selectedProcessEditor = value;
  }

  /**
   * Get process editor model which contains the process stored in the file filePath on the server.
   */
  public function getOpenProcessByFilePath(process:FileNavigatorProcess):ProcessEditorModel {
    var result:ProcessEditorModel = null;
    for each (var processEditorModel:ProcessEditorModel in openedProcesses) {
      if (process == processEditorModel.fileNavigatorProcess) {
        result = processEditorModel;
        break;
      }
    }
    return result;
  }

  public function get fileNavigatorModel():FileNavigatorModel {
    return _fileNavigatorModel;
  }

  public function get isPropertiesViewVisible():Boolean {
    return _isPropertiesViewVisible;
  }

  public function set isPropertiesViewVisible(value:Boolean):void {
    if (_isPropertiesViewVisible == value) {
      return;
    }
    _persistentData.data.selected_propView = value;
    _persistentData.flush();
    _isPropertiesViewVisible = value;

  }

  public function get isMicroImageViewVisible():Boolean {
    return _isMicroImageViewVisible;
  }

  public function set isMicroImageViewVisible(value:Boolean):void {
    if (_isMicroImageViewVisible == value) {
      return;
    }
    _persistentData.data.selected_micro = value;
    _persistentData.flush();
    _isMicroImageViewVisible = value;

  }

  public function get orchestraConnectionProperties():OrchestraConnectionPropertiesModel {
    return _orchestraConnectionProperties;
  }

  public function get accessProcessSettings():AccessProcessSettingsModel {
    return _accessProcessSettings;
  }

  public function set orchestraConnectionProperties(value:OrchestraConnectionPropertiesModel):void {
    if (value.jmxObjectName != orchestraConnectionProperties.jmxObjectName) {
      _persistentData.data.jmxObjectName = value.jmxObjectName;
      _persistentData.flush();
    }
    if (value.jmxServiceURL != orchestraConnectionProperties.jmxServiceURL) {
      _persistentData.data.jmxServiceURL = value.jmxServiceURL;
      _persistentData.flush();
    }
    _orchestraConnectionProperties = value;
  }

  public function get userName():String {
    return _userName;
  }

  public function set userName(name:String):void {
    _userName = name;
  }

  public function get usersOfDesigner():ArrayCollection {
    return _usersOfDesigner;
  }

  public function get unreadNotifications():ArrayCollection {
    return _unreadNotifications;
  }
}
}
