/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.model {
import mx.collections.ArrayCollection;

import org.ow2.orchestra.designer.vo.FileNavigatorItem;
import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
import org.ow2.orchestra.designer.vo.FileNavigatorProject;

[Bindable]
public class FileNavigatorModel {

  private var _projects:ArrayCollection;

  private var _selectedItem:FileNavigatorItem;

  // binding with corresponding variable in the FileNavigatorView
  private var _isEnabledDoubleClick:Boolean = true;

  public function FileNavigatorModel() {
    _projects = new ArrayCollection();
  }


  public function get projects():ArrayCollection {
    return _projects;
  }

  /**
   * Get project with the same name if already exist into the FileNavigatorView model.
   */
  public function getProject(projectToLook:FileNavigatorProject):FileNavigatorProject {
    for each (var project:FileNavigatorProject in projects) {
      if (projectToLook.name == project.name && projectToLook.owner.name == project.owner.name) {
        return project;
      }
    }
    return null;
  }

  /**
   * Get process with the same name if already exist into the FileNavigatoreView model.
   */
  public function getProcess(processToLook:FileNavigatorProcess):FileNavigatorProcess {
    if (processToLook != null && processToLook.project != null) {
      var project:FileNavigatorProject = getProject(processToLook.project);
      if (project != null) {
        for each (var process:FileNavigatorProcess in project.processes) {
          if (processToLook.name == process.name) {
            return process;
          }
        }
      }
    }
    return null;
  }

  /**
   * Verify if a projectName's already exist into the FileNavigatoreView model.
   * @param newProject the new project name to be created
   * @return true if the project already exist, false else
   *
   */
  public function projectExist(newProject:String):Boolean {
    for each (var project:FileNavigatorProject in projects) {
      if (newProject == project.name) {
        return true;
      }
    }
    return false;
  }

  /**
   * Find into project if the process name's exist
   * @param project the parent project
   * @param newProcess
   * @return
   *
   */
  public function processExist(project:FileNavigatorProject, newProcess:String):Boolean {
    for each (var process:FileNavigatorProcess in project.processes) {
      if (newProcess == process.name) {
        return true;
      }
    }
    return false;
  }

  public function get selectedItem():FileNavigatorItem {
    return _selectedItem;
  }

  public function set selectedItem(value:FileNavigatorItem):void {
    _selectedItem = value;
  }

  public function get isEnabledDoubleClick():Boolean {
    return _isEnabledDoubleClick;
  }

  public function set isEnabledDoubleClick(value:Boolean):void {
    _isEnabledDoubleClick = value;
  }

}
}
