/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.accessProperties {
import flash.events.Event;
import flash.events.MouseEvent;

import mx.collections.ArrayCollection;
import mx.collections.ListCollectionView;
import mx.containers.Form;
import mx.containers.HBox;
import mx.containers.TitleWindow;
import mx.containers.VBox;
import mx.controls.Button;
import mx.controls.ComboBox;
import mx.controls.HRule;
import mx.controls.Label;
import mx.controls.Spacer;
import mx.controls.TextInput;
import mx.core.Application;
import mx.events.CloseEvent;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;
import mx.events.PropertyChangeEvent;
import mx.managers.PopUpManager;

import org.ow2.orchestra.designer.events.AccessPropertiesEvent;
import org.ow2.orchestra.designer.events.CollectionAddAllEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.utils.ErrorManager;
import org.ow2.orchestra.designer.view.ConfirmPopup;
import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
import org.ow2.orchestra.designer.vo.FileNavigatorUserAccessPermissions;

public class AccessPropertyView extends TitleWindow {
  private var _dataGrid:AccessProcessSettingsDataGrid;

  private var text:TextInput;
  private var processSelector:ComboBox;


  public function AccessPropertyView() {
    super();
    _dataGrid = new AccessProcessSettingsDataGrid();
    _dataGrid.percentHeight = 100;
    _dataGrid.percentWidth = 100;
    _dataGrid.id = "lAccess";
    _dataGrid.dataProvider = new ArrayCollection();

    DesignerModelLocator.getInstance().accessProcessSettings.lAccess.addEventListener(CollectionEvent.COLLECTION_CHANGE,
        this.updateAccessList);
    DesignerModelLocator.getInstance().accessProcessSettings.lAccess.addEventListener(
        CollectionAddAllEvent.COLLECTION_CHANGE_ADD_ALL, this.updateAccessList);

    createUI();

    this.title = "Access Properties Settings";
    DesignerModelLocator.getInstance().accessProcessSettings.addEventListener(PropertyChangeEvent.PROPERTY_CHANGE,
        this.updateTitle);

    this.automationName = "processConfigWindow";

  }

  private function updateAccessList(event:CollectionEvent):void {
    var list:ArrayCollection = new ArrayCollection();
    if (event.kind == CollectionEventKind.ADD && event is CollectionAddAllEvent) {
      list.addAll(ListCollectionView(_dataGrid.dataProvider));
      list.addAll(new ArrayCollection(event.items));
    } else if (event.kind == CollectionEventKind.REMOVE) {
      list.addAll(ListCollectionView(_dataGrid.dataProvider));
      for each (var permission:FileNavigatorUserAccessPermissions in event.items) {
        list.removeItemAt(list.getItemIndex(permission));
      }
    } else if (event.kind != CollectionEventKind.RESET) {
      return;
    }
    _dataGrid.dataProvider = list;
    _dataGrid.modified = false;
  }

  public function dispose():void {
    DesignerModelLocator.getInstance().accessProcessSettings.lAccess.removeEventListener(CollectionEvent.COLLECTION_CHANGE,
        this.updateAccessList);
    DesignerModelLocator.getInstance().accessProcessSettings.lAccess.removeEventListener(
        CollectionAddAllEvent.COLLECTION_CHANGE_ADD_ALL, this.updateAccessList);
    DesignerModelLocator.getInstance().accessProcessSettings.removeEventListener(PropertyChangeEvent.PROPERTY_CHANGE,
        this.updateTitle);
    DesignerModelLocator.getInstance().accessProcessSettings.processesWithAdminPermission.removeEventListener(
        CollectionEvent.COLLECTION_CHANGE, this.processesWithAdminPermissionChanged);
    DesignerModelLocator.getInstance().accessProcessSettings.processesWithAdminPermission.removeEventListener(
        CollectionAddAllEvent.COLLECTION_CHANGE_ADD_ALL, this.processesWithAdminPermissionChanged);
  }

  private function updateTitle(event:PropertyChangeEvent):void {
    if (event.property == "selectedProcess" && DesignerModelLocator.getInstance().accessProcessSettings.selectedProcess != null) {
      this.title = "Access Properties Settings of " + DesignerModelLocator.getInstance().accessProcessSettings.selectedProcess.name;
    }
  }

  public function createUI():void {
    var vbox:VBox = new VBox();
    vbox.automationName = "processConfigBox";
    this.addChild(_dataGrid);
    this.addChild(vbox);
    var form:Form = new Form();

    var hbox1:HBox = new HBox();
    hbox1.width = 500;
    var lab:Label = new Label();
    lab.text = "New user";
    text = new TextInput();
    text.automationName = "newUser";

    var lab1:Label = new Label();
    lab1.text = "Select one opened process:";
    processSelector = new ComboBox();
    processSelector.automationName = "processSelector";
    processSelector.labelFunction = function(itemObj:FileNavigatorProcess):String {
      if (itemObj.project.owner.name == DesignerModelLocator.getInstance().userName) {
        return (itemObj.project.name + "/" + itemObj.name );
      } else {
        return (itemObj.project.name + "/" + itemObj.name + " by " + itemObj.project.owner.name);
      }
    };
    processSelector.addEventListener(Event.CHANGE, changeHandler);
    processSelector.maxWidth = 350;
    //Bind selector and processesWithAdminPermission
    DesignerModelLocator.getInstance().accessProcessSettings.processesWithAdminPermission.addEventListener(
        CollectionEvent.COLLECTION_CHANGE, processesWithAdminPermissionChanged);
    DesignerModelLocator.getInstance().accessProcessSettings.processesWithAdminPermission.addEventListener(
        CollectionAddAllEvent.COLLECTION_CHANGE_ADD_ALL, processesWithAdminPermissionChanged);
    // Refresh processesWithAdminPermission list
    new AccessPropertiesEvent(AccessPropertiesEvent.FETCH_ADMIN_PROCESSES).dispatch();

    var hbox2:HBox = new HBox();
    hbox2.width = 500;
    var addBT:Button = new Button();
    addBT.label = "Add user";
    addBT.automationName = "addUser";
    addBT.addEventListener(MouseEvent.CLICK, addNewEntry);

    var space1:Spacer = new Spacer();
    space1.width = 15;

    hbox1.addChild(lab);
    hbox1.addChild(text);
    hbox1.addChild(addBT);
    hbox1.addChild(space1);

    hbox2.addChild(lab1);
    hbox2.addChild(processSelector);

    form.addChild(hbox1);
    form.addChild(hbox2);

    var rule:HRule = new HRule();
    rule.width = 500;

    var hbox3:HBox = new HBox();

    var confirmBT:Button = new Button();
    confirmBT.label = "Save";
    confirmBT.automationName = "save";
    confirmBT.addEventListener(MouseEvent.CLICK, onConfirmHandle);

    var confirmAndCloseBT:Button = new Button();
    confirmAndCloseBT.label = "Save and close";
    confirmAndCloseBT.automationName = "saveClose";
    confirmAndCloseBT.addEventListener(MouseEvent.CLICK, onSaveAndCloseHandle);

    var cancelBT:Button = new Button();
    cancelBT.label = "Cancel";
    cancelBT.automationName = "cancel";
    cancelBT.addEventListener(MouseEvent.CLICK, onCancelHandle);
    var space2:Spacer = new Spacer();
    var space3:Spacer = new Spacer();
    hbox3.addChild(confirmBT);
    hbox3.addChild(space2);
    hbox3.addChild(confirmAndCloseBT);
    hbox3.addChild(space3);
    hbox3.addChild(cancelBT);

    hbox3.setStyle("horizontalAlign", "center");
    hbox3.setStyle("paddingBottom", 10);
    hbox3.setStyle("paddingTop", 10);
    hbox3.setStyle("paddingLeft", 10);
    hbox3.setStyle("paddingRight", 10);

    vbox.addChild(form);
    vbox.addChild(rule);
    vbox.addChild(hbox3);
  }

  private function processesWithAdminPermissionChanged(event:CollectionEvent):void {
    var processes:ArrayCollection = new ArrayCollection();
    if (event.kind == CollectionEventKind.ADD && event is CollectionAddAllEvent) {
      processes.addAll(ListCollectionView(processSelector.dataProvider));
      processes.addAll(new ArrayCollection(event.items));
    } else if (event.kind == CollectionEventKind.REMOVE) {
      processes.addAll(ListCollectionView(processSelector.dataProvider));
      for each (var process:FileNavigatorProcess in event.items) {
        processes.removeItemAt(processes.getItemIndex(process));
      }
    } else if (event.kind != CollectionEventKind.RESET) {
      return;
    }
    processSelector.dataProvider = processes;

    if (processes.length > 0) {
      //First check if a process was previously selected
      //Then check if the currently opened process is administered by the user
      //Otherwise select the first process in administered processes list
      var itemToSelect:FileNavigatorProcess = null;
      if (DesignerModelLocator.getInstance().accessProcessSettings.selectedProcess != null) {
        itemToSelect = listContainsProcess(processes, DesignerModelLocator.getInstance().accessProcessSettings.selectedProcess);
        if (itemToSelect != null) {
          //Next lines will fire a Change event on the ComboBox
          processSelector.selectedItem = itemToSelect;
          changeItem();
          return;
        }
      } else {
        if (DesignerModelLocator.getInstance().selectedProcessEditor != null) {
          itemToSelect =
              listContainsProcess(processes, DesignerModelLocator.getInstance().selectedProcessEditor.fileNavigatorProcess);
          if (itemToSelect != null) {
            //Next lines will fire a Change event on the ComboBox
            processSelector.selectedItem = itemToSelect;
            changeItem();
            return;
          }
        }
      }
      //Next lines will fire a Change event on the ComboBox
      processSelector.selectedItem = processes.getItemAt(0);
      changeItem();
    } else {
      processSelector.selectedIndex = -1;
    }
  }

  private function listContainsProcess(processes:ListCollectionView, process:FileNavigatorProcess):FileNavigatorProcess {
    for each (var fileNavigatorProcess:FileNavigatorProcess in processes) {
      if (fileNavigatorProcess.project.owner.name == process.project.owner.name &&
          fileNavigatorProcess.name == process.name &&
          fileNavigatorProcess.project.name == process.project.name) {
        return fileNavigatorProcess;
      }
    }
    return null;
  }

  private function onConfirmHandle(event:Event):void {
    // throw the event to save the modification of access permission
    if (_dataGrid.modified) {
      new AccessPropertiesEvent(AccessPropertiesEvent.ACCESS_PERMISSIONS_CHANGED,
          DesignerModelLocator.getInstance().accessProcessSettings.selectedProcess,
          _dataGrid.dataProvider as ListCollectionView).dispatch();
      _dataGrid.modified = false;
    }

    // Refresh processesWithAdminPermission list
    //new AccessPropertiesEvent(AccessPropertiesEvent.FETCH_ADMIN_PROCESSES).dispatch();
  }

  /**
   *  save and close the Popup of managing access permission
   * @param event
   */
  private function onSaveAndCloseHandle(event:Event):void {
    if (_dataGrid.modified) {
      new AccessPropertiesEvent(AccessPropertiesEvent.ACCESS_PERMISSIONS_CHANGED,
          DesignerModelLocator.getInstance().accessProcessSettings.selectedProcess,
          _dataGrid.dataProvider as ListCollectionView).dispatch();
      _dataGrid.modified = false;
    }
    this.dispose();
    PopUpManager.removePopUp(this);
    new AccessPropertiesEvent(AccessPropertiesEvent.PROCESS_CHANGED, null).dispatch();
  }


  private function onCancelHandle(event:Event):void {
    _dataGrid.modified = false;
    this.dispose();
    PopUpManager.removePopUp(this);
    new AccessPropertiesEvent(AccessPropertiesEvent.PROCESS_CHANGED, null).dispatch();
  }


  private function addNewEntry(event:MouseEvent):void {
    var user:String = text.text;
    text.text = null;
    if (user == null || user == "") {
      ErrorManager.getInstance().updateTitle("Error: empty user");
      ErrorManager.getInstance().showError(" failed to add user, the entry is null");
    } else if (checkExistentUser(user)) {
      ErrorManager.getInstance().updateTitle("Error: user already exists");
      ErrorManager.getInstance().showError(" User addition failed, the user already exists");
    } else if (!isUserOfDesigner(user)) {
      ErrorManager.getInstance().updateTitle("Error: user doesn't exist");
      ErrorManager.getInstance().showError(" User addition failed, the user doesn't exist");
    } else {
      var accessProps:ArrayCollection = new ArrayCollection();
      accessProps.addAll(ArrayCollection(_dataGrid.dataProvider));
      var access:FileNavigatorUserAccessPermissions = new FileNavigatorUserAccessPermissions();
      access.user = user;
      accessProps.addItem(access);
      _dataGrid.dataProvider = accessProps;
      _dataGrid.modified = true;
    }
  }

  private function isUserOfDesigner(userName:String):Boolean {
    for each (var user:String in DesignerModelLocator.getInstance().usersOfDesigner) {
      if (user == userName) {
        return true;
      }
    }
    return false;
  }

  private function checkExistentUser(user:String):Boolean {
    var result:Boolean = false;
    for each(var entryProperties:FileNavigatorUserAccessPermissions in DesignerModelLocator.getInstance().accessProcessSettings.lAccess) {
      if (entryProperties.user == user) {
        result = true;
      }
    }
    return result;
  }


  public function changeHandler(event:Event):void {
    var selectedProcess:FileNavigatorProcess = processSelector.selectedItem as FileNavigatorProcess;
    if (selectedProcess != DesignerModelLocator.getInstance().accessProcessSettings.selectedProcess) {
      if (_dataGrid.modified) {
        var warningWindow:ConfirmPopup = ConfirmPopup(PopUpManager.createPopUp(Application
            .application as Application, ConfirmPopup, true));
        warningWindow.title = "Warning : unsaved modifications";
        warningWindow.file.text = "Unsaved modifications will be lose";
        PopUpManager.centerPopUp(warningWindow);
        warningWindow.addEventListener(CloseEvent.CLOSE, itemNotSaveHandler);
        warningWindow.addEventListener(ConfirmPopup.CANCEL, itemNotSaveHandler);
      }
      else {
        changeItem();
      }
    }
  }


  public function itemNotSaveHandler(event:CloseEvent):void {
    if (event.type == CloseEvent.CLOSE) {
      changeItem();
    } else {
      processSelector.selectedItem = DesignerModelLocator.getInstance().accessProcessSettings.selectedProcess;
    }
  }

  public function changeItem():void {
    //Re-init data grid
    _dataGrid.dataProvider = new ArrayCollection();
    new AccessPropertiesEvent(AccessPropertiesEvent.PROCESS_CHANGED,
        processSelector.selectedItem as FileNavigatorProcess).dispatch();

    //Let a chance to refresh processesWithAdminPermission list
    //new AccessPropertiesEvent(AccessPropertiesEvent.FETCH_ADMIN_PROCESSES).dispatch();
  }

}
}
