/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.view.dataMappingEditor {
import flash.events.Event;
import flash.geom.Point;
import flash.geom.Rectangle;

import mx.binding.utils.ChangeWatcher;
import mx.collections.ArrayCollection;
import mx.collections.ListCollectionView;
import mx.core.Application;
import mx.core.ScrollPolicy;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;
import mx.events.DragEvent;
import mx.events.MenuEvent;
import mx.managers.DragManager;

import org.granite.collections.BasicMap;
import org.granite.collections.IMap;
import org.ow2.orchestra.designer.bpmn.model.data.AbstractDataModel;
import org.ow2.orchestra.designer.bpmn.model.data.AbstractExpressionElementModel;
import org.ow2.orchestra.designer.bpmn.model.data.DataMappingModel;
import org.ow2.orchestra.designer.bpmn.model.data.DataObjectModel;
import org.ow2.orchestra.designer.bpmn.model.data.ExpressionDataInputModel;
import org.ow2.orchestra.designer.bpmn.model.data.ExpressionDataObjectModel;
import org.ow2.orchestra.designer.bpmn.model.data.type.DataTypeModel;
import org.ow2.orchestra.designer.bpmn.model.data.type.SequenceType;
import org.ow2.orchestra.designer.bpmn.model.data.type.XSDElementModel;
import org.ow2.orchestra.designer.bpmn.model.process.AbstractBPMNElementWithData;
import org.ow2.orchestra.designer.bpmn.model.utils.DataUtil;
import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
import org.ow2.orchestra.designer.bpmn.views.IFlowNodeView;
import org.ow2.orchestra.designer.bpmn.views.data.DataMappingView;
import org.ow2.orchestra.designer.events.DiagramModificationEvent;
import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
import org.ow2.orchestra.designer.model.DataMappingEditorModel;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.ElementPaletteDnDModel;
import org.ow2.orchestra.designer.view.PatchedScrollableArrowMenu;
import org.ow2.orchestra.designer.view.diagramEditor.AbstractDiagramEditorView;
import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorFlowStateUtil;
import org.ow2.orchestra.designer.view.menuBar.MenuBarItem;
import org.ow2.orchestra.designer.view.menuBar.MenuBarItemWithChildren;
import org.ow2.orchestra.designer.view.palette.AbstractElementPalette;
import org.ow2.orchestra.designer.view.palette.datamapping.DataInputPaletteView;
import org.ow2.orchestra.designer.view.palette.datamapping.DataMappingPalette;
import org.ow2.orchestra.designer.view.palette.datamapping.DataObjectPaletteView;

/**
 * @author Loic Albertin
 */
public class DataMappingEditorView extends AbstractDiagramEditorView {

  public function DataMappingEditorView(dataMappingEditorModel:DataMappingEditorModel) {
    super(dataMappingEditorModel);

    this.percentHeight = 100;
    this.percentWidth = 100;
    this.automationName = "dataMappingEditorView";

    onDataMappingModelChange();

    ChangeWatcher.watch(dataMappingEditorModel, "dataMappingModel", onDataMappingModelChange);

  }

  override protected function createDiagramEditorFlowStateUtil():DiagramEditorFlowStateUtil {
    return new DataMappingEditorFlowStateUtil(this);
  }

  private function onDataMappingModelChange(event:Event = null):void {
    if (_diagramView != null) {
      this.removeChild(_diagramView);
      DataMappingModel(_diagramView.flowModel).elements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, collectionChangeHandler);
    }

    dataMappingEditorModel.dataMappingModel.elements.addEventListener(
        CollectionEvent.COLLECTION_CHANGE, collectionChangeHandler);

    _diagramView = new DataMappingView(dataMappingEditorModel.dataMappingModel);
    _diagramView.percentHeight = 100;
    _diagramView.percentWidth = 100;
    this.addChildAt(_diagramView, 0);
  }


  protected override function diagramViewDragEnter(event:DragEvent):void {
    this.dragEnterValid = false;
    this.acceptDrop = false;
    // Accept object
    if (event.dragInitiator is AbstractElementPalette &&
        AbstractElementPalette(event.dragInitiator).parentPalette is DataMappingPalette) {
      DragManager.acceptDragDrop(this);
      DragManager.showFeedback(DragManager.COPY);
      this.acceptDrop = true;
      this.dragEnterValid = true;
    }
  }

  /** Object drag handler */
  protected override function diagramViewDragOver(event:DragEvent):void {
    if (dragEnterValid) {
      if (event.dragInitiator is AbstractElementPalette) {
        var elementRectangle:Rectangle;
        var elemDnDModel:ElementPaletteDnDModel = event.dragSource.dataForFormat("paletteItem") as ElementPaletteDnDModel;
        elementRectangle = elemDnDModel.image.getRect(this);
        if (elementRectangle.topLeft.x < 0
            || elementRectangle.topLeft.y < 0) {
          DragManager.showFeedback(DragManager.NONE);
          acceptDrop = false;
        } else {
          DragManager.showFeedback(DragManager.COPY);
          acceptDrop = true;
          //Check collisions
          for each (var elementView:AbstractElementView in diagramView.getChildren()) {
            if ((elementView is IFlowNodeView) &&
                IFlowNodeView(elementView).getCollisionRectangle(this).intersects(elementRectangle)) {
              DragManager.showFeedback(DragManager.NONE);
              acceptDrop = false;
              return;
            }
          }
        }
      }
    }
  }


  protected override function diagramViewDrop(event:DragEvent):void {
    if (acceptDrop) {
      var targetX:int = event.localX;
      var targetY:int = event.localY;
      if (event.dragSource.hasFormat("paletteItem")) {
        var elemDnDModel:ElementPaletteDnDModel = event.dragSource.dataForFormat("paletteItem") as ElementPaletteDnDModel;
        targetX -= elemDnDModel.deltaX;
        targetY -= elemDnDModel.deltaY;
      }
      var dataModel:AbstractExpressionElementModel = null;
      // get the dropped object
      if (event.dragInitiator is DataObjectPaletteView) {
        dataModel = new ExpressionDataObjectModel();
      } else if (event.dragInitiator is DataInputPaletteView) {
        dataModel = new ExpressionDataInputModel();
      }
      if (dataModel != null) {
        new DiagramModificationEvent(DiagramModificationEvent.DIAGRAM_ELEMENT_ADD,
            diagramEditorModel, dataModel, targetX, targetY).dispatch();
      }
    }
  }

  private function handleMenuClick(menuEvent:MenuEvent):void {
    var menuBarItem:MenuBarItem = menuEvent.item as MenuBarItem;
    var property:String = "";
    if (menuBarItem.models[0] is ExpressionDataObjectModel) {
      property = "dataObjectModel";
    } else if (menuBarItem.models[0] is ExpressionDataInputModel) {
      property = "dataInputModel";
    }
    var childNodeSelector:String = null;
    if (menuBarItem.models.length == 3) {
      childNodeSelector = String(menuBarItem.models[2]);
    }
    //Change selector
    new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED, this.dataMappingEditorModel,
        menuBarItem.models[0], "childNodeSelector", childNodeSelector).dispatch();

    new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED, this.diagramEditorModel,
        menuBarItem.models[0], property, menuBarItem.models[1]).dispatch();
  }

  private function createMenu(dataModel:AbstractExpressionElementModel, exprModels:ListCollectionView):void {
    var dataListProvider:ListCollectionView = new ArrayCollection();

    var targetX:int = dataModel.x;
    var targetY:int = dataModel.y;
    var elementView:AbstractElementView = this._diagramView.getElementViewByModel(dataModel);
    if (elementView != null) {
      targetX += elementView.width;
    }
    var dropPoint:Point = new Point(targetX, targetY);
    var map:IMap = new BasicMap();
    for each (var abstractExprElemModel:AbstractExpressionElementModel in exprModels) {
      var abstractDataModel:AbstractDataModel;
      if (abstractExprElemModel is ExpressionDataInputModel) {
        abstractDataModel = ExpressionDataInputModel(abstractExprElemModel).dataInputModel;
      } else {
        abstractDataModel = ExpressionDataObjectModel(abstractExprElemModel).dataObjectModel;
      }

      if (abstractExprElemModel.childNodeSelector == null) {
        if (!map.containsKey(abstractDataModel.name)) {
          var simpleMenuItem:MenuBarItem = new MenuBarItem(abstractDataModel.name, true, this.handleMenuClick,
              null, null, false, "normal", [dataModel, abstractDataModel]);
          map.put(abstractDataModel.name, simpleMenuItem);
        }
      } else {
        var parentMenuItem:MenuBarItem = map.get(abstractDataModel.name) as MenuBarItem;
        var inlineParent:Boolean = false;
        if (parentMenuItem != null && !(parentMenuItem is MenuBarItemWithChildren)) {
          inlineParent = true;
        }
        if (parentMenuItem == null || !(parentMenuItem is MenuBarItemWithChildren)) {
          parentMenuItem = new MenuBarItemWithChildren(abstractDataModel.name, true);
          map.put(abstractDataModel.name, parentMenuItem);

          if (inlineParent) {
            //Set parent element as a child of itself as parent menu can't be clicked
            MenuBarItemWithChildren(parentMenuItem).children.addItem(
                new MenuBarItem(abstractDataModel.name, true, this.handleMenuClick,
                    null, null, false, "normal", [dataModel, abstractDataModel]));
            MenuBarItemWithChildren(parentMenuItem).children.addItem(
                new MenuBarItem(abstractDataModel.name, true, null,
                    null, null, false, "separator"));
          }
        }

        var childMenuItem:MenuBarItem = new MenuBarItem(
            abstractDataModel.name + "." +
                abstractExprElemModel.childNodeSelector.substring(
                    abstractExprElemModel.childNodeSelector.lastIndexOf("}") + 1),
            true, this.handleMenuClick,
            null, null, false, "normal", [dataModel, abstractDataModel, abstractExprElemModel.childNodeSelector]);
        MenuBarItemWithChildren(parentMenuItem).children.addItem(childMenuItem);
      }
    }
    dataListProvider.addAll(map.values);
    var menu:PatchedScrollableArrowMenu = PatchedScrollableArrowMenu.createMenu(this, dataListProvider);
    menu.verticalScrollPolicy = ScrollPolicy.OFF;
    menu.arrowScrollPolicy = ScrollPolicy.AUTO;
    menu.scrollSpeed = 120;
    menu.addEventListener(MenuEvent.ITEM_CLICK, function(menuEvent:MenuEvent):void {
      var menuBarItem:MenuBarItem = menuEvent.item as MenuBarItem;
      if (menuBarItem != null) {
        if (menuBarItem.handler != null) {
          menuBarItem.handler(menuEvent);
        }
      }

    });
    //Ensure that there is at least 60px to display the menu
    //This should be enough to display both up and down arrows and one element
    var globalDropPoint:Point = this.localToGlobal(dropPoint);
    var menuYDelta:int = Application(Application.application).height - globalDropPoint.y;
    if (menuYDelta < 60) {
      globalDropPoint.y -= 60 - menuYDelta;
    }
    menu.show(globalDropPoint.x, globalDropPoint.y);
  }

  private function collectionChangeHandler(event:CollectionEvent):void {
    if (event.kind == CollectionEventKind.ADD &&
        event.items.length == 1 &&
        event.items[0] is AbstractExpressionElementModel) {
      var abstractExprElemModel:AbstractExpressionElementModel = AbstractExpressionElementModel(event.items[0]);
      //By default select first element of each kind
      var collection:ListCollectionView;
      if (abstractExprElemModel is ExpressionDataObjectModel) {
        collection = new ArrayCollection();
        for each (var dataObjectModel:DataObjectModel in DesignerModelLocator.getInstance().selectedProcessEditor.processModel.dataObjects.values) {
          var parentExprDataObj:ExpressionDataObjectModel = new ExpressionDataObjectModel();
          parentExprDataObj.dataObjectModel = dataObjectModel;
          collection.addItem(parentExprDataObj);
          if (dataObjectModel.type is SequenceType ||
              dataObjectModel.type is XSDElementModel && XSDElementModel(dataObjectModel.type).type is SequenceType) {
            var sequenceElements:ListCollectionView;
            if (dataObjectModel.type is SequenceType) {
              sequenceElements = SequenceType(dataObjectModel.type).elements
            } else {
              sequenceElements = SequenceType(XSDElementModel(dataObjectModel.type).type).elements;
            }
            for each (var dataTypeModel:DataTypeModel in sequenceElements) {
              var childExprDataObj:ExpressionDataObjectModel = new ExpressionDataObjectModel();
              childExprDataObj.dataObjectModel = dataObjectModel;
              childExprDataObj.childNodeSelector = dataTypeModel.QName;
              collection.addItem(childExprDataObj);
            }
          }
        }
        var firstDataObject:ExpressionDataObjectModel = ExpressionDataObjectModel(collection.getItemAt(0));
        ExpressionDataObjectModel(abstractExprElemModel).dataObjectModel = firstDataObject.dataObjectModel;
        ExpressionDataObjectModel(abstractExprElemModel).childNodeSelector = firstDataObject.childNodeSelector;
      } else if (abstractExprElemModel is ExpressionDataInputModel) {
        var elementWithData:AbstractBPMNElementWithData =
            AbstractBPMNElementWithData(DesignerModelLocator.getInstance().selectedProcessEditor.selectedElements.getItemAt(0));
        collection = DataUtil.getListOfDataInputsNotInDataMappingModel(elementWithData.dataInputs,
            dataMappingEditorModel.dataMappingModel);
        var firstDataInput:ExpressionDataInputModel = ExpressionDataInputModel(collection.getItemAt(0));
        ExpressionDataInputModel(abstractExprElemModel).dataInputModel = firstDataInput.dataInputModel;
        ExpressionDataInputModel(abstractExprElemModel).childNodeSelector = firstDataInput.childNodeSelector;
      }
      if (collection.length > 1) {
        //Call later menu creation in order to let data mapping view create view corresponding to the abstractExprElemModel
        callLater(createMenu, [abstractExprElemModel, collection]);
      }
    }
  }

  public function dispose():void {
    if (_diagramView != null) {
      DataMappingModel(_diagramView.flowModel).elements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, collectionChangeHandler);
    }
  }

  public function get dataMappingEditorModel():DataMappingEditorModel {
    return DataMappingEditorModel(this.diagramEditorModel);
  }


}
}
