/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


package org.ow2.orchestra.designer.view.diagramEditor {
import flash.display.DisplayObject;
import flash.events.MouseEvent;
import flash.geom.Point;

import mx.collections.ArrayCollection;
import mx.events.FlexEvent;
import mx.states.SetEventHandler;
import mx.states.State;

import org.ow2.orchestra.designer.bpmn.model.FlowModel;
import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
import org.ow2.orchestra.designer.bpmn.views.FlowView;
import org.ow2.orchestra.designer.bpmn.views.process.ProcessPoolView;

public class DiagramEditorDefaultStateHelper extends DiagramEditorStateHelper {

  public function DiagramEditorDefaultStateHelper(diagramEditorView:AbstractDiagramEditorView) {
    super(diagramEditorView);
  }


  /**
   * Generate default state.
   * In the default state:
   * - the selection rectangle is hidden
   * - on mouse down on the processView canvas, this view goes to the "selecting" state.
   */
  public override function addStates():void {
    var state:State = new State();
    state.name = "default";
    diagramEditorView.states.push(state);

    // handler for mouse down event
    var mouseDownListener:SetEventHandler = new SetEventHandler(diagramEditorView, MouseEvent.MOUSE_DOWN);
    mouseDownListener.handlerFunction = defaultStateMouseDown;
    state.overrides.push(mouseDownListener);

    state.addEventListener(FlexEvent.ENTER_STATE, defaultStateEnter);
  }

  /**
   * Show context panel when entering default state if only one element is selected.
   * @param event
   */
  private function defaultStateEnter(event:FlexEvent):void {
    diagramEditorView.showElementContextPanel();
  }


  /**
   * handler for mouse down event when this view is in the "default" state.
   * On mouse down on a element or on the canvas, this view goes to the "selecting" state.
   * On mouse down on a selected element, this view goes to the "moving" state.
   */
  private function defaultStateMouseDown(event:MouseEvent):void {


    // Get mouse event position. The coordinates are relative to the target element
    var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
    // Transform the coordinates to be relative to this view
    var mousePoint:Point = diagramEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

    // set the selectionRect position
    diagramEditorView.selectionRect.x = mousePoint.x;
    diagramEditorView.selectionRect.y = mousePoint.y;
    // reset the selectionRect size
    diagramEditorView.selectionRect.width = 1;
    diagramEditorView.selectionRect.height = 1;

    if (diagramEditorView.diagramEditorModel.selectedElements.length == 1 && diagramEditorView.diagramEditorModel.selectedElements.getItemAt(0) is FlowModel) {
      var flowView:FlowView = diagramEditorView.diagramView.getElementViewByModel(diagramEditorView.diagramEditorModel
          .selectedElements.getItemAt(0) as FlowModel) as FlowView;
      for each (var displayObject:DisplayObject in flowView.getChildren()) {
        if (displayObject is FlowSelectionAnchor) {
          if (displayObject.getRect(diagramEditorView).containsPoint(mousePoint)) {
            if (FlowSelectionAnchor(displayObject).isSource) {
              diagramEditorView.currentState = "movingSourceFlow";
            } else {
              diagramEditorView.currentState = "movingTargetFlow";
            }
            break;
          }
        }
      }
    }
    if (diagramEditorView.currentState == "default") {
      // Ignore event if the mouse in over the context panel
      // Mouse is over the context panel if
      // - the context panel is in default state and the context panel rect contains the mouse point and the element rect does not contain the mouse point
      // - the context panel is in another state (e.g. remove state balloon) and the context panel rect contains the mouse point
      if (!diagramEditorView.elementContextPanel.visible
          || !diagramEditorView.elementContextPanel.getRect(diagramEditorView).containsPoint(mousePoint)
          || (diagramEditorView.elementContextPanel.currentState == "default"
          && diagramEditorView.elementContextPanel.attachedView.containsPoint(mousePoint, diagramEditorView)
          && !(diagramEditorView.elementContextPanel.attachedView is FlowView))) {
        // check if mouse is on a element view or on the canvas.
        // find list of elements that are contained in the selection area
        var listSelectedElements:ArrayCollection = new ArrayCollection();
        for each (var object:DisplayObject in diagramEditorView.diagramView.getChildren()) {
          if (!(object is ProcessPoolView) && object is AbstractElementView) {
            if (AbstractElementView(object).containsPoint(mousePoint, diagramEditorView)) {
              // the process element view is contained in the selection area,
              // the attached model should be selected.
              listSelectedElements.addItem((object as AbstractElementView).model);
            }
          }
        }
        if (listSelectedElements.length != 0
            && diagramEditorView.diagramEditorModel.selectedElements.contains(listSelectedElements.getItemAt(0))) {
          // if mouse is on a selected element go to "moving" state
          // go to move state
          diagramEditorView.currentState = "moving";
        } else {
          // go to "selecting" state
          diagramEditorView.currentState = "selecting";
        }
      }
    }
  }
}
}
