/**
 * Created by ${PRODUCT_NAME}.
 * User: camillej
 * Date: 24/02/11
 * Time: 13:59
 * To change this template use File | Settings | File Templates.
 */
package org.ow2.orchestra.designer.view.diagramEditor {
import flash.display.DisplayObject;
import flash.events.MouseEvent;
import flash.geom.Point;

import mx.collections.ArrayCollection;
import mx.managers.CursorManager;
import mx.styles.StyleManager;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.FlowModel;
import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
import org.ow2.orchestra.designer.bpmn.views.process.ProcessPoolView;

public class DiagramEditorFlowStateUtil {

  protected var diagramEditorView:AbstractDiagramEditorView;

  private static var copyCursor:Class = StyleManager.getStyleDeclaration('DragManager').getStyle('copyCursor');
  private static var rejectedCursor:Class = StyleManager.getStyleDeclaration('DragManager').getStyle('rejectCursor');

  public function DiagramEditorFlowStateUtil(diagramEditorView:AbstractDiagramEditorView) {
    this.diagramEditorView = diagramEditorView;
  }

  /**
   * Restore default cursor.
   */
  public function resetCursor():void {
    CursorManager.removeCursor(CursorManager.currentCursorID);
  }

  /**
   *
   * <p>Checks if elements under mouse are allowed to be target of a sequence flow.</p>
   * <p>This implementation check if there is one and only one element under mouse and if this element is not also the source of the sequence flow.</p>
   *
   * @param mouseEvent MouseEvent for moving event
   * @return true if element under mouse is allowed to be source of a sequence flow.
   */
  public function allowFlowTarget(mouseEvent:MouseEvent):Boolean {
    var listSelectedElements:ArrayCollection = new ArrayCollection();
    listSelectedElements = getElementUnderMouse(mouseEvent);

    if (listSelectedElements.length != 1) {
      return false;
    }
    // only one selected element.
    var selectedElement:AbstractElementWithPosition = listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
    var elementSource:AbstractElement =
        diagramEditorView.diagramEditorModel.selectedElements.getItemAt(0) as AbstractElement;
    var flowModel:FlowModel = null;
    if (elementSource != null && elementSource is FlowModel) {
      flowModel = elementSource as FlowModel;
      elementSource = flowModel.sourceElement;
    }
    return elementSource != selectedElement;
  }

  /**
   *
   * <p>Checks if elements under mouse are allowed to be source of a sequence flow.</p>
   * <p>This implementation check if there is one and only one element under mouse and if this element is not also the target of the sequence flow.</p>
   *
   * @param mouseEvent MouseEvent for moving event
   * @return true if element under mouse is allowed to be source of a sequence flow.
   */
  public function allowFlowSource(mouseEvent:MouseEvent):Boolean {
    var listSelectedElements:ArrayCollection = new ArrayCollection();
    listSelectedElements = getElementUnderMouse(mouseEvent);
    if (listSelectedElements.length != 1) {
      return false;
    }
    // only one selected element.
    var selectedElement:AbstractElementWithPosition = listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
    var elementTarget:AbstractElement =
        diagramEditorView.diagramEditorModel.selectedElements.getItemAt(0) as AbstractElement;
    var flowModel:FlowModel = null;
    if (elementTarget != null && elementTarget is FlowModel) {
      flowModel = elementTarget as FlowModel;
      elementTarget = flowModel.targetElement;
    }
    return elementTarget != selectedElement;
  }

  /**
   * Update mouse cursor with copy or rejected cursor if the drop is accepted/rejected
   * @param acceptDrop is drop allowed ?
   */
  public function updateDropMouseCursor(acceptDrop:Boolean):void {
    // Delete cursor used during drag operations
    resetCursor();

    if (acceptDrop) {
      // change cursor with drop ok
      CursorManager.setCursor(copyCursor);
    } else {
      CursorManager.setCursor(rejectedCursor);
    }
  }

  /**
   * Gets the list of AbstractElementWithPosition elements view currently under the mouse pointer.
   * @param mouseEvent
   * @return
   */
  public function getElementUnderMouse(mouseEvent:MouseEvent):ArrayCollection {
    var listSelectedElements:ArrayCollection = new ArrayCollection();
    if (mouseEvent.target != null) {
      // Get mouse event position. The coordinates are relative to the target element
      var targetLocalMousePoint:Point = new Point(mouseEvent.localX, mouseEvent.localY);
      // Transform the coordinates to be relative to this view
      var mousePoint:Point = diagramEditorView.globalToLocal(DisplayObject(mouseEvent.target).localToGlobal(targetLocalMousePoint));

      // check if mouse is on a element view or on the canvas.
      // find list of elements that are contained under the mouse cursor
      for each (var object:DisplayObject in diagramEditorView.diagramView.getChildren()) {
        if (!(object is ProcessPoolView) && object is AbstractElementView && AbstractElementView(object).model is AbstractElementWithPosition) {
          // get the child object display zone (with coordinates relative to this view)
          if (AbstractElementView(object).containsPoint(mousePoint, diagramEditorView)) {
            // the process element view is contained in the selection area,
            // the attached model should be selected.
            listSelectedElements.addItem((object as AbstractElementView).model);
          }
        }
      }
    }
    return listSelectedElements;
  }
}
}
