/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.diagramEditor {
import flash.display.DisplayObject;
import flash.events.MouseEvent;
import flash.geom.Point;

import mx.collections.ArrayCollection;
import mx.events.FlexEvent;
import mx.states.SetEventHandler;
import mx.states.State;

import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.FlowModel;
import org.ow2.orchestra.designer.bpmn.views.FlowView;
import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;

public class DiagramEditorMovingFlowStateHelper extends DiagramEditorStateHelper {

  protected var _flowStateUtil:DiagramEditorFlowStateUtil;

  public function DiagramEditorMovingFlowStateHelper(diagramEditorView:AbstractDiagramEditorView, flowStateUtil:DiagramEditorFlowStateUtil) {
    super(diagramEditorView);
    _flowStateUtil = flowStateUtil;
  }


  override public function addStates():void {
    this.addMovingSourceState();
    this.addMovingTargetState()
  }

  public function addMovingSourceState():void {
    var state:State = new State();
    state.name = "movingSourceFlow";
    diagramEditorView.states.push(state);

    var mouseMoveListener:SetEventHandler = new SetEventHandler(diagramEditorView, MouseEvent.MOUSE_MOVE);
    mouseMoveListener.handlerFunction = this.handleMouseMoveSource;
    state.overrides.push(mouseMoveListener);

    var mouseClickListener:SetEventHandler = new SetEventHandler(diagramEditorView, MouseEvent.MOUSE_UP);
    mouseClickListener.handlerFunction = this.handleMouseUpSource;
    state.overrides.push(mouseClickListener);
    state.addEventListener(FlexEvent.EXIT_STATE, movingStateExit);
  }

  public function addMovingTargetState():void {
    var state:State = new State();
    state.name = "movingTargetFlow";
    diagramEditorView.states.push(state);

    var mouseMoveListener:SetEventHandler = new SetEventHandler(diagramEditorView, MouseEvent.MOUSE_MOVE);
    mouseMoveListener.handlerFunction = this.handleMouseMoveTarget;
    state.overrides.push(mouseMoveListener);

    var mouseClickListener:SetEventHandler = new SetEventHandler(diagramEditorView, MouseEvent.MOUSE_UP);
    mouseClickListener.handlerFunction = this.handleMouseUpTarget;
    state.overrides.push(mouseClickListener);
    state.addEventListener(FlexEvent.EXIT_STATE, movingStateExit);
  }

  private function movingStateExit(event:FlexEvent):void {
    var flowView:FlowView = diagramEditorView.diagramView.getElementViewByModel(diagramEditorView.diagramEditorModel
        .selectedElements.getItemAt(0) as FlowModel) as FlowView;
    // when exiting moving states Redraw sequence flow
    //This handles mouse out events and click on non-abstract element with position
    flowView.redraw(event);

    _flowStateUtil.resetCursor();
  }


  private function handleMouseMoveSource(mouseEvent:MouseEvent):void {
    this.handleMouseMove(mouseEvent, true);
  }

  private function handleMouseUpSource(mouseEvent:MouseEvent):void {
    this.handleMouseUp(mouseEvent, true);
  }

  private function handleMouseMoveTarget(mouseEvent:MouseEvent):void {
    this.handleMouseMove(mouseEvent, false);
  }

  private function handleMouseUpTarget(mouseEvent:MouseEvent):void {
    this.handleMouseUp(mouseEvent, false);
  }

  protected function handleMouseMove(mouseEvent:MouseEvent, isSource:Boolean):void {
    diagramEditorView.hideElementContextPanel();

    // Get mouse event position. The coordinates are relative to the target element
    var targetLocalMousePoint:Point = new Point(mouseEvent.localX, mouseEvent.localY);
    // Transform the coordinates to be relative to this view
    var mousePoint:Point = diagramEditorView.globalToLocal(DisplayObject(mouseEvent
        .target).localToGlobal
        (targetLocalMousePoint));
    // A sequence flow view should be selected
    var flowView:FlowView = diagramEditorView.diagramView.getElementViewByModel(diagramEditorView.diagramEditorModel
        .selectedElements.getItemAt(0) as FlowModel) as FlowView;
    if (isSource) {
      flowView.sourcePoint = mousePoint;
      _flowStateUtil.updateDropMouseCursor(_flowStateUtil.allowFlowSource(mouseEvent));
    } else {
      flowView.targetPoint = mousePoint;
      _flowStateUtil.updateDropMouseCursor(_flowStateUtil.allowFlowTarget(mouseEvent));
    }
  }

  private function handleMouseUp(mouseEvent:MouseEvent, isSource:Boolean):void {
    // Check if object under mouse is a valid target
    if ((isSource && _flowStateUtil.allowFlowSource(mouseEvent))
        || (!isSource && _flowStateUtil.allowFlowTarget(mouseEvent))) {
      var listSelectedElements:ArrayCollection = new ArrayCollection();
      listSelectedElements = _flowStateUtil.getElementUnderMouse(mouseEvent);
      var flowView:FlowView = diagramEditorView.diagramView.getElementViewByModel(diagramEditorView.diagramEditorModel
          .selectedElements.getItemAt(0) as FlowModel) as FlowView;

      if (listSelectedElements.length == 1) {
        //Dispatch creating flow event in order to create flow from source element to target element
        var targetElement:AbstractElementWithPosition =
            listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
        if (isSource) {
          new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED,
              diagramEditorView.diagramEditorModel, flowView.model, "sourceElement",
              targetElement).dispatch();
        } else {
          new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED,
              diagramEditorView.diagramEditorModel, flowView.model, "targetElement",
              targetElement).dispatch();
        }
      }
    }
    diagramEditorView.currentState = "default";

  }
}

}
