/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.fileNavigator {
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.events.Event;
import flash.events.KeyboardEvent;
import flash.events.MouseEvent;
import flash.ui.Keyboard;

import mx.binding.utils.BindingUtils;
import mx.collections.ArrayCollection;
import mx.collections.ICollectionView;
import mx.collections.ListCollectionView;
import mx.collections.Sort;
import mx.collections.SortField;
import mx.controls.Tree;
import mx.core.Application;
import mx.core.ClassFactory;
import mx.events.CloseEvent;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;
import mx.events.FlexEvent;
import mx.events.ListEvent;
import mx.managers.CursorManager;
import mx.managers.PopUpManager;

import org.ow2.orchestra.designer.events.FileNavigatorEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.view.ConfirmPopup;
import org.ow2.orchestra.designer.vo.FileNavigatorItem;
import org.ow2.orchestra.designer.vo.FileNavigatorOwner;
import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
import org.ow2.orchestra.designer.vo.FileNavigatorProject;

/**
 * The file navigator view.
 */
public class FileNavigatorView extends Tree {
  /**
   * The source for "project" image data binding.
   */
  [Bindable]
  [Embed(source="/../assets/icon/accordion/project.gif")]
  public static var projectIcon:Class;

  /**
   * The source for "Bpmn" image data binding.
   */
  [Bindable]
  [Embed(source="/../assets/icon/accordion/bpmn.gif")]
  public static var bpmnFileIcon:Class;

  private var deleteConfirmWindow:ConfirmPopup;

  private var _isEnabledDoubleClick:Boolean = true;
  private var sharedProcessesTreeItem:ParentTreeItem;

  /**
   * Initialize the file navigator view.
   */
  public function FileNavigatorView() {
    super();

    var sharedProcesses:FileNavigatorProject = new FileNavigatorProject();
    var owner:FileNavigatorOwner = new FileNavigatorOwner();
    owner.name = DesignerModelLocator.getInstance().userName;
    owner.projects.addItem(sharedProcesses);
    sharedProcesses.owner = owner;
    sharedProcesses.name = "Shared Processes";
    sharedProcessesTreeItem = new ParentTreeItem(sharedProcesses);

    this.x = 0;
    this.y = 0;
    this.percentHeight = 100;
    this.percentWidth = 100;
    this.iconFunction = iconFun;

    this.showRoot = true;
    this.labelField = "label";
    this.dragEnabled = true;
    this.dropEnabled = false;

    this.itemRenderer = new ClassFactory(SharedProcessesTreeItemRenderer);

    this.dataProvider = new ArrayCollection();

    var sort:Sort = new Sort();
    sort.fields = [new SortField("label", true)];
    ICollectionView(super.dataProvider).sort = sort;
    //apply sort or findItem will throw exception...
    ICollectionView(super.dataProvider).refresh();

    this.doubleClickEnabled = true;

    CursorManager.setBusyCursor();
    Application.application.addEventListener(FlexEvent.APPLICATION_COMPLETE, initFiles);
    this.initEventListener();
  }

  private function iconFun(item:Object):Class {
    if (item.itemData is FileNavigatorProject) {
      return projectIcon;
    }
    else if (item.itemData is FileNavigatorProcess) {
      return bpmnFileIcon;
    }
    return null;
  }

  private function initEventListener():void {
    DesignerModelLocator.getInstance().fileNavigatorModel.projects.addEventListener(
        CollectionEvent.COLLECTION_CHANGE, dataProviderChangeHandler);

    this.addEventListener(MouseEvent.DOUBLE_CLICK, this.tree_itemDoubleClick);
    this.addEventListener(ListEvent.ITEM_CLICK, this.tree_itemClick);
    this.addEventListener(KeyboardEvent.KEY_DOWN, this.itemDelete);
    BindingUtils.bindProperty(this, "selectedItem", DesignerModelLocator.getInstance().fileNavigatorModel, ["selectedItem"]);
    BindingUtils.bindProperty(this, "isEnabledDoubleClick", DesignerModelLocator.getInstance().fileNavigatorModel,
        ["isEnabledDoubleClick"])
  }


  /**
   * Handler of doubleClick event in a item of the tree control
   * If the current Item selected is a project expand the node
   * Else if the current Item selected is a process then open it into editor
   */
  public function tree_itemDoubleClick(event:MouseEvent):void {

    if (this.selectedItem == null) {
      return;
    }
    var node:Object = TreeItem(this.selectedItem).itemData;
    if (node != null && node is FileNavigatorProcess) {
      this.fileOpen();
    }
  }

  /**
   * Handler of click event in a item of the tree control
   * Sets the model selected item
   */
  public function tree_itemClick(event:ListEvent):void {
    if (this.selectedItem == null) {
      return;
    }
    DesignerModelLocator.getInstance().fileNavigatorModel.selectedItem =
        TreeItem(this.selectedItem).itemData as FileNavigatorItem;
  }

  /**
   * Remove a item in the parent item with the suppr keyboard
   */
  public function itemDelete(event:KeyboardEvent):void {
    if (event.keyCode == Keyboard.DELETE && this.selectedItem != null) {

      // remove temporarily keydown listener
      this.removeEventListener(KeyboardEvent.KEY_DOWN, this.itemDelete);

      var type:String;
      var fileItem:FileNavigatorItem = TreeItem(this.selectedItem).itemData as FileNavigatorItem;
      if (fileItem is FileNavigatorProject) {
        type = "project";
      }
      else {
        type = "process";
      }

      deleteConfirmWindow = ConfirmPopup(PopUpManager.createPopUp(Application.application as Application, ConfirmPopup, true));
      deleteConfirmWindow.title = "Warning : " + type + " deletion...";

      if (fileItem is FileNavigatorProject) {
        deleteConfirmWindow.file.text = "Do you want to delete " + type + " '" + fileItem.name + "' (and all included " +
            "processes)?\rThis operation can't be undone.";
      } else {
        deleteConfirmWindow.file.text = "Do you want to delete " + type + " '" + fileItem.name + "'?\rThis operation can't " +
            "be undone.";
      }
      PopUpManager.centerPopUp(deleteConfirmWindow);
      deleteConfirmWindow.setFocus();
      deleteConfirmWindow.addEventListener(CloseEvent.CLOSE, itemDeleteHandler);
      deleteConfirmWindow.addEventListener(ConfirmPopup.CANCEL, itemDeleteHandler);
    }
  }

  private function itemDeleteHandler(event:CloseEvent):void {
    if (event.type == CloseEvent.CLOSE) {
      if (TreeItem(this.selectedItem).itemData is FileNavigatorItem) {
        new FileNavigatorEvent(FileNavigatorEvent.ITEM_DELETE,
            TreeItem(this.selectedItem).itemData as FileNavigatorItem).dispatch();
      }
    }
    //add keydown listener temporarily deleted during deleteConfirmWindow modal windows opening
    this.addEventListener(KeyboardEvent.KEY_DOWN, this.itemDelete);
  }

  /**
   * Open the current selected item in the FileNavigator view if possible (ie a process)
   *
   */
  public function fileOpen():void {
    var process:FileNavigatorProcess = TreeItem(this.selectedItem).itemData as FileNavigatorProcess;

    var evt:FileNavigatorEvent = new FileNavigatorEvent(FileNavigatorEvent.PROCESS_OPEN, process);
    CairngormEventDispatcher.getInstance().dispatchEvent(evt);
  }

  /**
   * Initialize the list of the files show into fileNavigatorView.
   * This is done during the CreationComplete phase, because the ServiceLocator elements are not initialized before.
   *
   */
  public function initFiles(event:Event):void {
    new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL).dispatch();
  }


  override public function set selectedItem(value:Object):void {
    if (value != null) {
      if (value is TreeItem) {
        if (value.itemData is FileNavigatorProcess) {
          this.expandItem(
              this.getItemForData(FileNavigatorProcess(value.itemData).project, this.dataProvider as ListCollectionView),
              true);
        }
        else if (value.itemData is FileNavigatorProject) {
          this.expandItem(value, true);
        }
        super.selectedItem = value;
      } else if (value is FileNavigatorItem) {
        this.selectedItem = this.getItemForData(value, this.dataProvider as ListCollectionView);
      }
    } else {
      super.selectedItem = null;
    }
  }

  /**
   * Recursively look into a collection of {@link TreeItem}s a given itemData object.
   * @param data the data to retrieve
   * @param list the  collection of {@link TreeItem}s
   * @return the {@link TreeItem} containing the given data or <code>null</code>
   */
  private function getItemForData(data:Object, list:ListCollectionView):TreeItem {
    for each (var treeItem:TreeItem in list) {
      if (treeItem.itemData == data) {
        return treeItem;
      }
      if (treeItem is ParentTreeItem) {
        var childItem:TreeItem = this.getItemForData(data, ParentTreeItem(treeItem).children);
        if (childItem != null) {
          return childItem;
        }
      }
    }
    return null;
  }

  /**
   * Disabled double click to prevent open twice the same process.
   *
   */
  public function set isEnabledDoubleClick(value:Boolean):void {
    _isEnabledDoubleClick = value;
    if (_isEnabledDoubleClick)
      this.addEventListener(MouseEvent.DOUBLE_CLICK, this.tree_itemDoubleClick);
    else
      this.removeEventListener(MouseEvent.DOUBLE_CLICK, this.tree_itemDoubleClick);
  }

  /**
   * Update data provider
   * @param event
   */
  private function dataProviderChangeHandler(event:CollectionEvent = null):void {
    if (event.kind == CollectionEventKind.ADD) {
      for each (var fileNavigatorItem:FileNavigatorItem in event.items) {
        if (fileNavigatorItem is FileNavigatorProject) {
          var children:ArrayCollection = new ArrayCollection();
          for each (var fileNavigatorProcess:FileNavigatorProcess in
              FileNavigatorProject(fileNavigatorItem).processes) {
            children.addItem(new TreeItem(fileNavigatorProcess));
          }
          if (FileNavigatorProject(fileNavigatorItem).owner.name != DesignerModelLocator.getInstance().userName) {
            if (sharedProcessesTreeItem.children.length == 0) {
              ListCollectionView(this.dataProvider).addItem(sharedProcessesTreeItem);
            }
            sharedProcessesTreeItem.children.addAll(children);
          } else {
            ListCollectionView(this.dataProvider).addItem(new ParentTreeItem(fileNavigatorItem, children));
          }
        }
      }
    } else if (event.kind == CollectionEventKind.REMOVE) {
      for each (var fileNavigatorProject:FileNavigatorProject in event.items) {
        if (fileNavigatorProject.owner.name != DesignerModelLocator.getInstance().userName) {
          for each (var process:FileNavigatorProcess in fileNavigatorProject.processes) {
            var processItem:TreeItem = this.getItemForData(process, this.dataProvider as ListCollectionView);
            sharedProcessesTreeItem.children.removeItemAt(sharedProcessesTreeItem.children.getItemIndex(processItem));
            if (sharedProcessesTreeItem.children.length == 0) {
              ListCollectionView(this.dataProvider).removeItemAt(
                  ListCollectionView(this.dataProvider).getItemIndex(sharedProcessesTreeItem));
            }
          }
        } else {
          var projectItem:TreeItem = this.getItemForData(fileNavigatorProject, this.dataProvider as ListCollectionView);
          ListCollectionView(this.dataProvider).removeItemAt(
              ListCollectionView(this.dataProvider).getItemIndex(projectItem));
        }
      }
    } else if (event.kind == CollectionEventKind.RESET) {
      sharedProcessesTreeItem.children.removeAll();
      ListCollectionView(this.dataProvider).removeAll();
    }
  }

  /**
   * Sort dataProvider returned by your super class
   */
  override public function get dataProvider():Object {
    var object:Object = super.dataProvider;
    //Remove shared processes if exists
    var index:int = ListCollectionView(object).getItemIndex(sharedProcessesTreeItem);
    if (index != -1) {
      ListCollectionView(object).removeItemAt(index);
    }
    if (ICollectionView(object).length > 0) {
      // Sort collection
      ICollectionView(object).refresh();
    }
    //if shared processes were present add them at the end of the list
    if (index != -1) {
      ListCollectionView(object).addItem(sharedProcessesTreeItem);
    }
    return object;
  }
}
}
