/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.palette {
import mx.containers.Box;
import mx.containers.BoxDirection;
import mx.controls.Image;
import mx.controls.Label;

public class AbstractElementPalette extends Box {

  protected var imageElement:Image;
  protected var imageSource:Class;

  protected var imageWidth:int;
  protected var imageHeight:int;

  // size of image elements include into the palette
  private static const DEFAULT_IMAGE_WIDTH:int = 30;
  private static const DEFAULT_IMAGE_HEIGHT:int = 30;

  private var _parentPalette:AbstractPalette;

  public function AbstractElementPalette(sourceImage:Class, labelText:String, parentPalette:AbstractPalette,
                                         width:int = DEFAULT_IMAGE_WIDTH,
                                         height:int = DEFAULT_IMAGE_HEIGHT,
                                         boxDirection:String = BoxDirection.HORIZONTAL) {
    imageWidth = width;
    imageHeight = height;
    imageSource = sourceImage;
    imageElement = duplicateImage();
    _parentPalette = parentPalette;
    this.direction = boxDirection;
    this.setStyle("horizontalAlign", "center");

    this.addChild(imageElement);
    if (labelText != "") {
      var label:Label = new Label();
      label.text = labelText;
      this.addChild(label);
    }
  }


  public function duplicateImage():Image {
    var newImage:Image = new Image();
    newImage.source = imageSource;
    newImage.width = imageWidth;
    newImage.height = imageHeight;

    return newImage;
  }

  public function get parentPalette():AbstractPalette {
    return _parentPalette;
  }
}
}
