/**
 * @author Loic Albertin
 */
package org.ow2.orchestra.designer.view.palette.datamapping {
import mx.events.CollectionEvent;
import mx.events.PropertyChangeEvent;

import org.ow2.orchestra.designer.bpmn.model.process.AbstractBPMNElementWithData;
import org.ow2.orchestra.designer.bpmn.model.utils.DataUtil;

import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.view.palette.AbstractElementPalette;

public class DataInputPaletteView extends AbstractElementPalette {

  /**
   * The source for "OutputData" image data binding into the palette.
   */
  [Bindable]
  [Embed(source="/../assets/palette/dataInputDefault.svg")]
  public var defaultImgDataInput:Class;

  private var _watchedElementWithData:AbstractBPMNElementWithData = null;

  public function DataInputPaletteView(parentPalette:DataMappingPalette) {
    super(defaultImgDataInput, "", parentPalette, 20, 30);
    //We want to enable or disable this abstract element palette, so lets inspect the currently selected process editor model
    DesignerModelLocator.getInstance().addEventListener(PropertyChangeEvent.PROPERTY_CHANGE, propertyChangeHandler);
  }

  private function propertyChangeHandler(event:PropertyChangeEvent):void {
    if (event.property == "selectedProcessEditor") {
      //if there was a selected process editor remove listeners
      if (event.oldValue != null) {
        ProcessEditorModel(event.oldValue).selectedElements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, collectionChangeHandler);
        if (_watchedElementWithData != null && _watchedElementWithData.dataMappingModel != null) {
          _watchedElementWithData.dataMappingModel.elements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, dataMappingCollectionChangeHandler);
        }
      }
      //if there is a selected process editor add listeners
      if (event.newValue != null) {
        ProcessEditorModel(event.newValue).selectedElements.addEventListener(CollectionEvent.COLLECTION_CHANGE, collectionChangeHandler);
        collectionChangeHandler();
      }
    }
  }

  private function collectionChangeHandler(event:CollectionEvent = null):void {
    //if a element with data was monitored then remove listener
    if (_watchedElementWithData != null && _watchedElementWithData.dataMappingModel != null) {
      _watchedElementWithData.dataMappingModel.elements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, dataMappingCollectionChangeHandler);
    }

    if (DesignerModelLocator.getInstance().selectedProcessEditor.selectedElements.length == 1 &&
        DesignerModelLocator.getInstance().selectedProcessEditor.selectedElements.getItemAt(0) is
            AbstractBPMNElementWithData) {
      _watchedElementWithData =
          DesignerModelLocator.getInstance().selectedProcessEditor.selectedElements.getItemAt(0) as
              AbstractBPMNElementWithData;
      //add a listener on it's data mapping
      if (_watchedElementWithData.dataMappingModel != null) {
        _watchedElementWithData.dataMappingModel.elements.addEventListener(CollectionEvent.COLLECTION_CHANGE, dataMappingCollectionChangeHandler);
        dataMappingCollectionChangeHandler();
      }
    }
  }



  private function dataMappingCollectionChangeHandler(event:CollectionEvent = null):void {
    this.enabled = DataUtil.getListOfDataInputsNotInDataMappingModel(
        _watchedElementWithData.dataInputs, _watchedElementWithData.dataMappingModel).length > 0;
  }
}
}
