/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.properties {
import mx.controls.ComboBox;
import mx.controls.Label;
import mx.core.UIComponent;
import mx.events.ListEvent;

import org.granite.util.Enum;
import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;

[Bindable]
public class EnumProperty extends AbstractProperty {
  private var _itemEditor:ComboBox;
  private var _itemRenderer:Label;


  public function EnumProperty(propertyName:String, description:String, value:Enum, constants:Array) {
    super(propertyName, description);
    _itemRenderer = new Label();
    _itemRenderer.text = labelEnum(value as Enum);

    _itemEditor = new ComboBox();
    _itemEditor.labelFunction = labelEnum;
    _itemEditor.dataProvider = constants;
    _itemEditor.selectedItem = value;
    _itemEditor.addEventListener(ListEvent.CHANGE, this.valueChanged, false, 0, true);
    _itemEditor.automationName = propertyName + "Editor";
  }

  public override function get itemEditor():UIComponent {
    return _itemEditor;
  }


  public override function get itemRenderer():UIComponent {
    return _itemRenderer;
  }


  private function valueChanged(event:ListEvent):void {
    var element:AbstractElement = getSelectedElement();
    if (element == null) {
      return;
    }
    new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED,
        DesignerModelLocator.getInstance().selectedProcessEditor, element, _propertyName,
            _itemEditor.selectedItem).dispatch();
  }

  private function labelEnum(value:Enum):String {
    return value.name.replace("_", " ").toLowerCase();
  }
}
}